/*
 * Copyright (C) 1999  CyberPsychotic (fygrave@tigerteam.net)
 * http://www.kalug.lug.net/srsh/
 *	
 * Permission to use, copy, and modify this software without fee
 * is hereby granted, provided that this entire notice is included in
 * all copies of any software which is or includes a copy or
 * modification of this software. 
 * You may use this code under the GNU public license if you so wish. Please
 * contribute changes back to the authors under this freer than GPL license
 * so that we may further the use of strong encryption without limitations to
 * all.
 *
 * THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTY. IN PARTICULAR, NONE OF THE AUTHORS MAKES ANY
 * REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE
 * MERCHANTABILITY OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR
 * PURPOSE.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "srsh.h"

#define VERSION 	"0.1"
#define MAX_ARGS 	10
#define BUF_SIZE	1024

#define DEF_PATH "/bin:/usr/bin"
#define DEF_TERM "vt100"


char *bad_chars[]={
"..",
"./",
"/",
"\0"};

typedef struct _command {
	char name[BUF_SIZE];
	char path[BUF_SIZE];
	char descr[BUF_SIZE];
} command;

command allowed_commands[]={
{"ls","/bin/ls","list directories"},
{"cp","/bin/cp","copy files/directories"},
{"mv","/bin/mv","move directories"},
{"rm","/bin/rm","remove directories"},
{"pine","/usr/bin/pine","mailer"},
{"passwd","/usr/bin/passwd","change password"},
{"w","/usr/bin/w","show who is in system"},
{"who","/bin/who","same as w"},
{"gzip","/bin/gzip","archiver" },
{"gunzip","/bin/gunzip","dearchiver"},
{"finger","/bin/finger","similar to who"},
{"lynx","/usr/bin/lynx","web browser"},
{"env","/usr/bin/env","show envinronment"},
/* add your commands here */
{"\0","\0"}};

typedef struct _iternal_cmd {
	char name[BUF_SIZE];
	int (*funct)();
	char descr[BUF_SIZE];
} iternal_cmd;


iternal_cmd i_commands[] = {
{ "vt100",i_term,"set your TERMinal type to vt100"},
{ "exit",i_exit,"exit system"},
 { "cd",i_cd,"change working directory"},
 { "help",i_help,"shows this help"},
  { "\0",NULL}};


void initialize(void) {

printf("Simplified Restricted Shell v."VERSION"\n");
printf("coded by CyberPsychotic(fygrave@tigerteam.net)\n");
printf("http://www.kalug.lug.net/\n");

/* set defaults */
setenv("PATH",DEF_PATH,1);
setenv("TERM",DEF_TERM,0);

/* trap signals */

signal(SIGKILL,sig_handler);
signal(SIGTERM,sig_handler);
signal(SIGINT,sig_handler);
signal(SIGQUIT,sig_handler);
signal(SIGSTOP,sig_handler);
signal(SIGTSTP,sig_handler);

}

void put_banner(void) {
char pathname[MAXPATHLEN];
getcwd(pathname,MAXPATHLEN);
printf("[srsh: %s]$ ",pathname);
fflush(stdout);
}

void sig_handler(int sig) {

printf("sig: %i received\n",sig);
if(sig == SIGKILL || sig == SIGTERM || sig == SIGQUIT) 
				exit(1);
}




int i_term(char **args) {

setenv("TERM","vt100",1);
return 1;
}

int i_help(char **args) {

iternal_cmd *icptr;
command *cptr;
int i=0;


cptr=allowed_commands;
icptr=i_commands;

printf("\navailable commands:\n");
while(*cptr->name)  { 
		printf("%s - %s%s",cptr->name,
				cptr->descr,i++%2?"\n":"\t\t\t\t");
		cptr++;
}

while(*icptr->name)  {
		printf("%s - %s%s",icptr->name,
				icptr->descr,i++%2?"\n":"\t\t\t\t");
		icptr++;
}

printf("\n--== simplified restricted shell v."VERSION"==--\n");

return 1;
}

int i_exit(char **args) {

printf("Bye!\n");
fflush(stdout);
exit(1);
}

int i_cd(char **args) {

printf("cd is not allowed\n");
fflush(stdout);
return 1;
}

int is_iternal(char **args, int argnum) {
iternal_cmd *cptr;

cptr=i_commands;
while(*cptr[0].name) { 
	if(!strcmp(args[0],cptr[0].name)) { 
	cptr[0].funct(&args[1]);
	return 1;
	} /* if */
   cptr++;
   } /* while */

return 0;
}

int is_allowed(char **args, int argnum) {

command *cptr;
cptr=allowed_commands;
while(*cptr[0].name) { 
	if(!strcmp(args[0],cptr[0].name)) {
		args[0]=cptr[0].path;
		return 1;
	}

	if(!strcmp(args[0],cptr[0].path)) {
		args[0]=cptr[0].path;
		return 1;
	}
        cptr++;
	}

return 0;

}

int execute_command(char *command) {
char *tok;
char *args[MAX_ARGS];
int argnum;
int status;
pid_t pid;
#ifdef DEBUG
int i;
#endif

argnum=0;
tok=command;
bzero(args,sizeof(args));
#ifdef LINUX
/* linux has broken strsep, that's why we use strtok instead */
args[0]=strtok(tok," \r\n\a");
if(args[argnum] && *args[argnum]) argnum++;
#endif
while(tok!=NULL && argnum<MAX_ARGS) {
#ifdef LINUX
	args[argnum]=strtok(NULL," \r\n\a");
	tok=args[argnum];
	if(tok) argnum++;
#else
	args[argnum]=strsep(&tok," \r\n\a");
        if(args[argnum] && *args[argnum]) argnum++;
#endif
}
if(!argnum) return 1; /* empty string */
args[argnum]=NULL; /* terminator */
#ifdef DEBUG
for (i=0;i<argnum;i++)  printf("%i %s\n",i,args[i]);
#endif
/* if command is iternal */
if (is_iternal(args,argnum)) return 1;
/* if command is external but not allowed */
if (!is_allowed(args,argnum)) { 
		fprintf(stderr,"Permittion denied.Attempt logged\n");
		return 0;
	}

/* exec external */
pid=fork();
if (pid<0) { perror("can not fork"); return 0; }

if(pid>0) { /* parent */
	waitpid(pid,&status,0);
	if (WIFSIGNALED(status)) printf("process %i interrupted by signal\n",
									  pid);
	return 1;
}
/* else child here */
execvp(args[0],&args[0]);
perror("exec");
exit(0);
}

int main(int argc, char **argv) {

char	buf[BUF_SIZE];
	initialize();
	put_banner();
	while(fgets(buf,BUF_SIZE-1,stdin)) {
	if(strlen(buf)) execute_command(buf);
#ifdef DEBUG
	fprintf(stderr,"command executed\n");
#endif
	/* if(!execute_command(buf)) 
		printf("%s: woops..\n", argv[0]); 
	that's how you can log failed attempts.
	*/
	put_banner();
	}

return 0;
}
